# Generated by Django 2.0.7 on 2018-07-05 14:44

import django.db.models.deletion

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("friendship", "0002_block"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="block", unique_together={("blocker", "blocked")},
        ),
    ]
