<form method="post">
{% csrf_token %}
<input type="hidden" name="blocked_username" value="{{ blocked_username }}" />
<input type="submit" value="remove {{ blocked_username }} from blocklist" />
</form>
